@@ -71,6 +71,14 @@ void snd_compat_request_module(const char *name, ...);
 #ifndef __safe
 #define __safe
 #endif
+#include <linux/types.h>
+#ifndef __bitwise
+#define __bitwise
+typedef __u16 __le16;
+typedef __u16 __be16;
+typedef __u32 __le32;
+typedef __u32 __be32;
+#endif
 #endif /* < 2.6.9 */
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
@@ -115,6 +123,10 @@ void snd_compat_request_module(const char *name, ...);
 #ifndef __devexit_p
 #define __devexit_p(x) x
 #endif
+#ifndef __init_or_module
+#define __init_or_module
+#define __initdata_or_module
+#endif
 
 #include <linux/kdev_t.h>
 #ifndef major
@@ -199,13 +211,10 @@ static inline struct proc_dir_entry *PDE(const struct inode *inode)
 #endif
 #endif
 
-#include <linux/types.h>
-#ifndef __bitwise
-typedef __u16 __le16;
-typedef __u16 __be16;
-typedef __u32 __le32;
-typedef __u32 __be32;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 9)
+#define in_atomic()	in_interrupt()
 #endif
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 14)
 typedef unsigned __nocast gfp_t;
 #endif
@@ -521,6 +530,17 @@ enum {
 #define URB_NO_TRANSFER_DMA_MAP 0
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 3, 0) \
+    && LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 20)
+struct usb_ctrlrequest {
+	__u8 bRequestType;
+	__u8 bRequest;
+	__u16 wValue;
+	__u16 wIndex;
+	__u16 wLength;
+} __attribute__ ((packed));
+#endif
+
 #endif /* SND_NEED_USB_WRAPPER && CONFIG_USB */
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 24) \
@@ -642,7 +662,8 @@ static inline void *snd_compat_vmap(struct page **pages, unsigned int count, uns
 /* sysfs */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 2)
 struct class;
-static inline void class_device_create(struct class *class, struct class *parent, int devnum, ...) { return; }
+struct class_device;
+static inline struct class_device* class_device_create(struct class *class, struct class_device *parent, int devnum, ...) { return NULL; }
 static inline void class_device_destroy(struct class *class, int devnum) { return; }
 #else /* >= 2.6.2 */
 #ifndef CONFIG_SND_NESTED_CLASS_DEVICE
@@ -669,8 +690,8 @@ unsigned long snd_compat_msleep_interruptible(unsigned int msecs);
 /* msecs_to_jiffies */
 #ifndef CONFIG_HAVE_MSECS_TO_JIFFIES
 #include <linux/jiffies.h>
-#if defined(CONFIG_SUSE_KERNEL) && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
-#define HAVE_VARIABLE_HZ	/* HZ is a variable */
+#if defined(DESKTOP_HZ) && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
+#define HAVE_VARIABLE_HZ	/* 2.4 SUSE kernel, HZ is a variable */
 #endif
 static inline unsigned int jiffies_to_msecs(const unsigned long j)
 {
@@ -728,6 +749,22 @@ static inline unsigned long msecs_to_jiffies(const unsigned int m)
 #endif
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 17)
+#include <linux/dma-mapping.h>
+#ifndef DMA_31BIT_MASK
+#define DMA_31BIT_MASK	0x000000007fffffffULL
+#endif
+#ifndef DMA_30BIT_MASK
+#define DMA_30BIT_MASK	0x000000003fffffffULL
+#endif
+#ifndef DMA_28BIT_MASK
+#define DMA_28BIT_MASK	0x000000000fffffffULL
+#endif
+#ifndef DMA_24BIT_MASK
+#define DMA_24BIT_MASK	0x0000000000ffffffULL
+#endif
+#endif
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
 #include <linux/moduleparam.h>
 #undef module_param
@@ -778,29 +815,26 @@ int snd_pci_dev_present(const struct pci_device_id *ids);
 #endif
 #endif
 
-/* power management compatibility layer */
-#ifdef CONFIG_PM
-#ifdef PCI_OLD_SUSPEND
-void snd_card_pci_suspend(struct pci_dev *dev);
-void snd_card_pci_resume(struct pci_dev *dev);
-#define SND_PCI_PM_CALLBACKS \
-	.suspend = snd_card_pci_suspend,  .resume = snd_card_pci_resume
-#endif
-#endif
-
 /*
  * memory allocator wrappers
  */
 #ifdef CONFIG_SND_DEBUG_MEMORY
 
-void *snd_hidden_kmalloc(size_t size, unsigned int __nocast flags);
-void *snd_hidden_kzalloc(size_t size, unsigned int __nocast flags);
-void *snd_hidden_kcalloc(size_t n, size_t size, unsigned int __nocast flags);
-char *snd_hidden_kstrdup(const char *s, unsigned int __nocast flags);
+#include <linux/slab.h>
+void *snd_hidden_kmalloc(size_t size, gfp_t flags);
+void *snd_hidden_kzalloc(size_t size, gfp_t flags);
+void *snd_hidden_kcalloc(size_t n, size_t size, gfp_t flags);
+char *snd_hidden_kstrdup(const char *s, gfp_t flags);
 void snd_hidden_kfree(const void *obj);
 
-void *snd_wrapper_kmalloc(size_t size, unsigned int flags);
-void snd_wrapper_kfree(const void *obj);
+static inline void *snd_wrapper_kmalloc(size_t size, gfp_t flags)
+{
+	return kmalloc(size, flags);
+}
+static inline void snd_wrapper_kfree(const void *obj)
+{
+	kfree(obj);
+}
 
 #define kmalloc(size, flags) snd_hidden_kmalloc(size, flags)
 #define kzalloc(size, flags) snd_hidden_kzalloc(size, flags)
@@ -817,17 +851,17 @@ void snd_wrapper_kfree(const void *obj);
 #define kfree_nocheck(obj) kfree(obj)
 
 #ifndef CONFIG_HAVE_KCALLOC
-void *snd_compat_kcalloc(size_t n, size_t size, unsigned int __nocast gfp_flags);
+void *snd_compat_kcalloc(size_t n, size_t size, gfp_t gfp_flags);
 #define kcalloc(n,s,f) snd_compat_kcalloc(n,s,f)
 #endif
 
 #ifndef CONFIG_HAVE_KSTRDUP
-char *snd_compat_kstrdup(const char *s, unsigned int __nocast gfp_flags);
+char *snd_compat_kstrdup(const char *s, gfp_t gfp_flags);
 #define kstrdup(s,f) snd_compat_kstrdup(s,f)
 #endif
 
 #ifndef CONFIG_HAVE_KZALLOC
-void *snd_compat_kzalloc(size_t n, unsigned int __nocast gfp_flags);
+void *snd_compat_kzalloc(size_t n, gfp_t gfp_flags);
 #define kzalloc(s,f) snd_compat_kzalloc(s,f)
 #endif
 
@@ -846,7 +880,7 @@ void *snd_compat_kzalloc(size_t n, unsigned int __nocast gfp_flags);
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 11)
 #include <linux/pm.h>
 #ifndef PMSG_FREEZE
-typedef u32 /*__bitwise*/ pm_message_t;
+typedef u32 __bitwise pm_message_t;
 #define PMSG_FREEZE	3
 #define PMSG_SUSPEND	3
 #define PMSG_ON		0
@@ -866,6 +900,20 @@ typedef u32 /*__bitwise*/ pm_message_t;
 #endif
 #endif
 
+/* __GFP_XXX */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 6)
+/* #incldue <linux/gfp.h> */
+#ifndef __GFP_COMP
+#define __GFP_COMP	0
+#endif
+#ifndef __GFP_NOWARN
+#define __GFP_NOWARN	0
+#endif
+#ifndef __GFP_NORETRY
+#define __GFP_NORETRY	0
+#endif
+#endif
+
 /* vprintk */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 9)
 #include <linux/kernel.h>
@@ -897,26 +945,17 @@ struct snd_gameport {
 };
 static inline struct gameport *gameport_allocate_port(void)
 {
-	extern void *snd_hidden_kzalloc(size_t size, unsigned int flags);
 	struct snd_gameport *gp;
-#ifdef CONFIG_SND_DEBUG_MEMORY
-	gp = snd_hidden_kzalloc(sizeof(*gp), GFP_KERNEL);
-#else
-	gp = kcalloc(1, sizeof(*gp), GFP_KERNEL);
-#endif
+	gp = kzalloc(sizeof(*gp), GFP_KERNEL);
 	if (gp)
 		return &gp->gp;
 	return NULL;
 }
+#define gameport_free_port(gp)	kfree(gp)
 static inline void snd_gameport_unregister_port(struct gameport *gp)
 {
-	extern void snd_hidden_kfree(const void *obj);
 	gameport_unregister_port(gp);
-#ifdef CONFIG_SND_DEBUG_MEMORY
-	snd_hidden_kfree(gp);
-#else
 	kfree(gp);
-#endif
 }
 #undef gameport_unregister_port
 #define gameport_unregister_port(gp)	snd_gameport_unregister_port(gp)
@@ -933,6 +972,60 @@ static inline void snd_gameport_unregister_port(struct gameport *gp)
 #ifdef CONFIG_PCI
 #include <linux/pci.h>
 
+/* power management compatibility layer */
+#if defined(CONFIG_PM) && defined(PCI_OLD_SUSPEND)
+struct snd_compat_pci_driver {
+	struct pci_driver real_driver;
+	char *name;
+	const struct pci_device_id *id_table;
+	int (*probe)(struct pci_dev *dev, const struct pci_device_id *id);
+	void (*remove)(struct pci_dev *dev);
+	int (*suspend)(struct pci_dev *dev, u32 state);
+	int (*resume)(struct pci_dev *dev);
+};
+
+static inline void snd_pci_old_suspend(sturct pci_dev *pci)
+{
+	struct snd_compat_pci_driver *driver = (struct snd_compat_pci_driver *)pci->driver;
+	if (driver->suspend)
+		driver->suspend(pci, PMSG_SUSPEND);
+}
+static inline void snd_pci_old_resume(struct pci_dev *pci)
+{
+	struct snd_compat_pci_driver *driver = (struct snd_compat_pci_driver *)pci->driver;
+	if (driver->resume)
+		driver->resume(pci);
+}
+
+static inline int snd_pci_compat_register_driver(struct snd_compat_pci_driver *driver)
+{
+	driver->real_driver.name = driver->name;
+	driver->real_driver.id_table = driver->id_table;
+	driver->real_driver.probe = driver->probe;
+	driver->real_driver.remove = driver->remove;
+	if (driver->suspend || driver->resume) {
+		driver->real_driver.suspend = snd_pci_old_suspend;
+		driver->real_driver.resume = snd_pci_old_resume;
+	} else {
+		driver->real_driver.suspend = driver->suspend;
+		driver->real_driver.resume = driver->resume;
+	}
+	return pci_module_init(&driver->real_driver);
+}
+
+static inline void snd_pci_compat_unregister_driver(struct snd_compat_pci_driver *driver)
+{
+	pci_unregister_driver(&driver->real_driver);
+}
+
+#define pci_driver snd_compat_pci_driver
+#undef pci_register_driver
+#define pci_register_driver snd_pci_compat_register_driver
+#undef pci_unregister_driver
+#define pci_unregister_driver snd_pci_compat_unregister_driver
+
+#else
+
 static inline int snd_pci_compat_register_driver(struct pci_driver *driver)
 {
 	return pci_module_init(driver);
@@ -940,14 +1033,40 @@ static inline int snd_pci_compat_register_driver(struct pci_driver *driver)
 
 #undef pci_register_driver
 #define pci_register_driver snd_pci_compat_register_driver
-#endif
-#endif
+
+#endif /* CONFIG_PM && PCI_OLD_SUSPEND */
+#endif /* CONFIG_PCI */
+#endif /* 2.4 */
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 0)
+#ifdef CONFIG_PCI
+#ifndef CONFIG_HAVE_NEW_PCI_SAVE_STATE
+void snd_pci_compat_save_state(struct pci_dev *pci);
+void snd_pci_compat_restore_state(struct pci_dev *pci);
+static inline void snd_pci_orig_save_state(struct pci_dev *pci, u32 *buffer)
+{
+	pci_save_state(pci, buffer);
+}
+static inline void snd_pci_orig_restore_state(struct pci_dev *pci, u32 *buffer)
+{
+	pci_restore_state(pci, buffer);
+}
+
+#undef pci_save_state
+#define pci_save_state		snd_pci_compat_save_state
+#undef pci_restore_state
+#define pci_restore_state	snd_pci_compat_restore_state
+#endif /* !CONFIG_HAVE_NEW_PCI_SAVE_STATE */
+#endif /* CONFIG_PCI */
+#endif /* >= 2.4.0 */
 
 /* pci_get_device() and pci_dev_put() wrappers */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
+#ifdef CONFIG_PCI
 #define pci_get_device	pci_find_device
 #define pci_dev_put(x)
 #endif
+#endif
 
 /* wrapper for getnstimeofday()
  * it's needed for recent 2.6 kernels, too, due to lack of EXPORT_SYMBOL
@@ -968,4 +1087,91 @@ static inline int snd_pci_compat_register_driver(struct pci_driver *driver)
 #define schedule_timeout_uninterruptible(x) ({set_current_state(TASK_UNINTERRUPTIBLE); schedule_timeout(x);})
 #endif
 
+#if defined(CONFIG_PNP) && defined(CONFIG_PM)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
+#ifndef CONFIG_HAVE_PNP_SUSPEND
+#include <linux/pnp.h>
+#define HAVE_PNP_PM_HACK
+struct snd_pnp_driver {
+	struct pnp_driver real_driver;
+	char *name;
+	const struct pnp_device_id *id_table;
+	unsigned int flags;
+	int  (*probe)  (struct pnp_dev *dev, const struct pnp_device_id *dev_id);
+	void (*remove) (struct pnp_dev *dev);
+	int (*suspend) (struct pnp_dev *dev, pm_message_t state);
+	int (*resume) (struct pnp_dev *dev);
+};	
+
+int snd_pnp_register_driver(struct snd_pnp_driver *driver);
+static inline void snd_pnp_unregister_driver(struct snd_pnp_driver *driver)
+{
+	pnp_unregister_driver(&driver->real_driver);
+}
+
+#define pnp_driver	snd_pnp_driver
+#undef pnp_register_driver
+#define pnp_register_driver	snd_pnp_register_driver
+#undef pnp_unregister_driver
+#define pnp_unregister_driver	snd_pnp_unregister_driver
+
+struct snd_pnp_card_driver {
+	struct pnp_card_driver real_driver;
+	char * name;
+	const struct pnp_card_device_id *id_table;
+	unsigned int flags;
+	int  (*probe)  (struct pnp_card_link *card, const struct pnp_card_device_id *card_id);
+	void (*remove) (struct pnp_card_link *card);
+	int (*suspend) (struct pnp_card_link *dev, pm_message_t state);
+	int (*resume) (struct pnp_card_link *dev);
+};
+
+int snd_pnp_register_card_driver(struct snd_pnp_card_driver *driver);
+static inline void snd_pnp_unregister_card_driver(struct snd_pnp_card_driver *driver)
+{
+	pnp_unregister_card_driver(&driver->real_driver);
+}
+
+#define pnp_card_driver		snd_pnp_card_driver
+#undef pnp_register_card_driver
+#define pnp_register_card_driver	snd_pnp_register_card_driver
+#undef pnp_unregister_card_driver
+#define pnp_unregister_card_driver	snd_pnp_unregister_card_driver
+
+#endif /* ! CONFIG_HAVE_PNP_SUSPEND */
+#endif /* 2.6 */
+#endif /* CONFIG_PNP && CONFIG_PM */
+
+/*
+ * Another wrappers for pnp_register_*driver()
+ * They shouldn't return positive values in the new API
+ */
+#ifdef CONFIG_PNP
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0) && \
+	LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 17)
+#ifndef HAVE_PNP_PM_HACK
+#include <linux/pnp.h>
+static inline int snd_pnp_register_driver(struct pnp_driver *driver)
+{
+	int err = pnp_register_driver(driver);
+	return err < 0 ? err : 0;
+}
+#define pnp_register_driver	snd_pnp_register_driver
+
+static inline int snd_pnp_register_card_driver(struct pnp_card_driver *drv)
+{
+	int err = pnp_register_card_driver(drv);
+	return err < 0 ? err : 0;
+}
+#define pnp_register_card_driver	snd_pnp_register_card_driver
+
+#endif /* HAVE_PNP_PM_HACK */
+#endif /* < 2.6.17 */
+#endif /* PNP */
+
+/*
+ * old defines
+ */
+#define OPL3_HW_OPL3_PC98	0x0305	/* PC9800 */
+
 #endif /* __SOUND_LOCAL_DRIVER_H */
