@@ -96,9 +96,12 @@ static __inline__ void list_del_init(struct list_head *entry)
 /* Some distributions use modified kill_fasync */
 #ifdef CONFIG_OLD_KILL_FASYNC
 #include <linux/fs.h>
+static inline void snd_compat_kill_fasync(struct fasync_struct **fp, int sig, int band)
+{
+	kill_fasync(*(fp), sig);
+}
 #undef kill_fasync
-#define kill_fasync(fp, sig, band) snd_wrapper_kill_fasync(fp, sig, band)
-void snd_wrapper_kill_fasync(struct fasync_struct **, int, int);
+#define kill_fasync(fp, sig, band) snd_compat_kill_fasync(fp, sig, band)
 #endif
 
 /* this is identical with tq_struct but the "routine" field is renamed to "func" */
@@ -205,7 +208,11 @@ struct resource {
 #define IORESOURCE_MEM		0x00000200
 #endif
 
-void snd_wrapper_request_region(unsigned long from, unsigned long extent, const char *name);
+static inline void snd_wrapper_request_region(unsigned long from, unsigned long extent, const char *name)
+{
+	request_region(from, extent, name);
+}
+
 #undef request_region
 #define request_region(start,size,name) snd_compat_request_region(start,size,name,0)
 #define request_mem_region(start,size,name) snd_compat_request_region(start,size,name,1)
@@ -276,8 +283,8 @@ void snd_pci_compat_free_consistent(struct pci_dev *, long, void *, dma_addr_t);
 #define pci_request_regions(dev,name) snd_pci_compat_request_regions(dev,name)
 #define pci_release_regions(dev) snd_pci_compat_release_regions(dev)
 
-#define pci_save_state(dev,buf) snd_pci_compat_save_state(dev,buf)
-#define pci_restore_state(dev,buf) snd_pci_compat_restore_state(dev,buf)
+#define pci_save_state(dev) snd_pci_compat_save_state(dev)
+#define pci_restore_state(dev) snd_pci_compat_restore_state(dev)
 
 struct pci_device_id {
 	unsigned int vendor, device;		/* Vendor and device ID or PCI_ANY_ID */
@@ -290,6 +297,7 @@ struct pci_driver {
 	struct list_head node;
 	struct pci_dev *dev;
 	char *name;
+	void *owner;
 	const struct pci_device_id *id_table;	/* NULL if wants all devices */
 	int (*probe)(struct pci_dev *dev, const struct pci_device_id *id); /* New device inserted */
 	void (*remove)(struct pci_dev *dev);	/* Device removed (NULL if not a hot-plug capable driver) */
@@ -319,8 +327,8 @@ int snd_pci_compat_request_region(struct pci_dev *pdev, int bar, char *res_name)
 void snd_pci_compat_release_region(struct pci_dev *pdev, int bar);
 int snd_pci_compat_request_regions(struct pci_dev *pdev, char *res_name);
 void snd_pci_compat_release_regions(struct pci_dev *pdev);
-void snd_pci_compat_save_state(struct pci_dev *pdev, u32 *buf);
-void snd_pci_compat_restore_state(struct pci_dev *pdev, u32 *buf);
+void snd_pci_compat_save_state(struct pci_dev *pdev);
+void snd_pci_compat_restore_state(struct pci_dev *pdev);
 
 #define pci_module_init	snd_pci_compat_register_driver
 
@@ -575,4 +583,8 @@ static inline int abs(int val)
 	return (val < 0) ? -val : val;
 }
 
+#ifndef cpu_relax
+#define cpu_relax()
+#endif
+
 #endif /* <2.3.0 */
