@@ -31,20 +31,14 @@
 #error PAGE_SIZE is < 4k
 #endif
 
-static int restore_dsp_rettings(echoaudio_t *chip);
+static int restore_dsp_rettings(struct echoaudio *chip);
 
 
-//===========================================================================
-//
-// Some vector commands involve the DSP reading or writing data to and
-// from the comm page; if you send one of these commands to the DSP,
-// it will complete the command and then write a non-zero value to
-// the Handshake field in the comm page.  This function waits for the
-// handshake to show up.
-//
-//===========================================================================
-
-static int wait_handshake(echoaudio_t *chip)
+/* Some vector commands involve the DSP reading or writing data to and from the
+comm page; if you send one of these commands to the DSP, it will complete the
+command and then write a non-zero value to the Handshake field in the
+comm page.  This function waits for the handshake to show up. */
+static int wait_handshake(struct echoaudio *chip)
 {
 	int i;
 
@@ -64,19 +58,11 @@ static int wait_handshake(echoaudio_t *chip)
 
 
 
-//===========================================================================
-//
-// Much of the interaction between the DSP and the driver is done via vector
-// commands; send_vector writes a vector command to the DSP.  Typically,
-// this causes the DSP to read or write fields in the comm page.
-//
-// * PCI posting is not required thanks to the handshake logic
-//
-// Returns 0 if sent OK.
-//
-//===========================================================================
-
-static int send_vector(echoaudio_t *chip, u32 command)
+/* Much of the interaction between the DSP and the driver is done via vector
+commands; send_vector writes a vector command to the DSP.  Typically, this
+causes the DSP to read or write fields in the comm page.
+PCI posting is not required thanks to the handshake logic. */
+static int send_vector(struct echoaudio *chip, u32 command)
 {
 	int i;
 
@@ -98,14 +84,9 @@ static int send_vector(echoaudio_t *chip, u32 command)
 
 
 
-//===========================================================================
-//
-// write_dsp writes a 32-bit value to the DSP; this is used almost
-// exclusively for loading the DSP.
-//
-//===========================================================================
-
-static int write_dsp(echoaudio_t *chip, u32 data)
+/* write_dsp writes a 32-bit value to the DSP; this is used almost
+exclusively for loading the DSP. */
+static int write_dsp(struct echoaudio *chip, u32 data)
 {
 	u32 status, i;
 
@@ -127,14 +108,9 @@ static int write_dsp(echoaudio_t *chip, u32 data)
 
 
 
-//===========================================================================
-//
-// read_dsp reads a 32-bit value from the DSP; this is used almost
-// exclusively for loading the DSP and checking the status of the ASIC.
-//
-//===========================================================================
-
-static int read_dsp(echoaudio_t *chip, u32 *data)
+/* read_dsp reads a 32-bit value from the DSP; this is used almost
+exclusively for loading the DSP and checking the status of the ASIC. */
+static int read_dsp(struct echoaudio *chip, u32 *data)
 {
 	u32 status, i;
 
@@ -156,30 +132,22 @@ static int read_dsp(echoaudio_t *chip, u32 *data)
 
 
 /****************************************************************************
-
 	Firmware loading functions
-
  ****************************************************************************/
 
-//===========================================================================
-//
-// This function is used to read back the serial number from the DSP;
-// this is triggered by the SET_COMMPAGE_ADDR command.
-//
-// Only some early Echogals products have serial numbers in the ROM;
-// the serial number is not used, but you still need to do this as
-// part of the DSP load process.
-//
-//===========================================================================
-
-static int read_sn(echoaudio_t *chip)
+/* This function is used to read back the serial number from the DSP;
+this is triggered by the SET_COMMPAGE_ADDR command.
+Only some early Echogals products have serial numbers in the ROM;
+the serial number is not used, but you still need to do this as
+part of the DSP load process. */
+static int read_sn(struct echoaudio *chip)
 {
 	int i;
 	u32 sn[6];
 
 	for (i = 0; i < 5; i++) {
 		if (read_dsp(chip, &sn[i])) {
-			snd_printk((KERN_ERR "Failed to read serial number\n"));
+			snd_printk(KERN_ERR "Failed to read serial number\n");
 			return -EIO;
 		}
 	}
@@ -191,7 +159,7 @@ static int read_sn(echoaudio_t *chip)
 
 #ifndef ECHOCARD_HAS_ASIC
 /* This card has no ASIC, just return ok */
-static inline int check_asic_status(echoaudio_t *chip)
+static inline int check_asic_status(struct echoaudio *chip)
 {
 	chip->asic_loaded = TRUE;
 	return 0;
@@ -203,8 +171,8 @@ static inline int check_asic_status(echoaudio_t *chip)
 
 #ifdef ECHOCARD_HAS_ASIC
 
-// Load ASIC code - done after the DSP is loaded
-static int load_asic_generic(echoaudio_t *chip, u32 cmd, const struct firmware *asic)
+/* Load ASIC code - done after the DSP is loaded */
+static int load_asic_generic(struct echoaudio *chip, u32 cmd, const struct firmware *asic)
 {
 	const struct firmware *fw;
 	int err;
@@ -246,21 +214,12 @@ la_error:
 
 
 
-//===========================================================================
-//
-// install_resident_loader
-//
-// Install the resident loader for 56361 DSPs;  The resident loader
-// is on the EPROM on the board for 56301 DSP.
-//
-// The resident loader is a tiny little program that is used to load
-// the real DSP code.
-//
-//===========================================================================
-
 #ifdef DSP_56361
 
-static int install_resident_loader(echoaudio_t *chip)
+/* Install the resident loader for 56361 DSPs;  The resident loader is on
+the EPROM on the board for 56301 DSP. The resident loader is a tiny little
+program that is used to load the real DSP code. */
+static int install_resident_loader(struct echoaudio *chip)
 {
 	u32 address;
 	int index, words, i;
@@ -286,21 +245,13 @@ static int install_resident_loader(echoaudio_t *chip)
 		return i;
 	}
 
-	//---------------------------------------------------------------------------
-	//
-	// Loader
-	//
-	// The DSP code is an array of 16 bit words.  The array is divided up into
-	// sections.  The first word of each section is the size in words, followed
-	// by the section type.
-	//
-	// Since DSP addresses and data are 24 bits wide, they each take up two
-	// 16 bit words in the array.
-	//
-	// This is a lot like the other loader loop, but it's not a loop,
-	// you don't write the memory type, and you don't write a zero at the end.
-	//
-	//---------------------------------------------------------------------------
+	/* The DSP code is an array of 16 bit words.  The array is divided up
+	into sections.  The first word of each section is the size in words,
+	followed by the section type.
+	Since DSP addresses and data are 24 bits wide, they each take up two
+	16 bit words in the array.
+	This is a lot like the other loader loop, but it's not a loop, you
+	don't write the memory type, and you don't write a zero at the end. */
 
 	/* Set DSP format bits for 24 bit mode */
 	set_dsp_register(chip, CHI32_CONTROL_REG, get_dsp_register(chip, CHI32_CONTROL_REG) | 0x900);
@@ -369,15 +320,7 @@ irl_error:
 #endif /* DSP_56361 */
 
 
-//===========================================================================
-//
-// load_dsp
-//
-// This loads the DSP code.
-//
-//===========================================================================
-
-static int load_dsp(echoaudio_t *chip, u16 *code)
+static int load_dsp(struct echoaudio *chip, u16 *code)
 {
 	u32 address, data;
 	int index, words, i;
@@ -421,11 +364,9 @@ static int load_dsp(echoaudio_t *chip, u16 *code)
 	/* Set DSP format bits for 24 bit mode now that soft reset is done */
 	set_dsp_register(chip, CHI32_CONTROL_REG, get_dsp_register(chip, CHI32_CONTROL_REG) | 0x900);
 
-	//---------------------------------------------------------------------------
-	// Main loader loop
-	//---------------------------------------------------------------------------
-	index = code[0];
+	/* Main loader loop */
 
+	index = code[0];
 	for (;;) {
 		int block_type, mem_type;
 
@@ -495,9 +436,9 @@ static int load_dsp(echoaudio_t *chip, u16 *code)
 			}
 
 			/* Get the serial number via slave mode.
-			 * This is triggered by the SET_COMMPAGE_ADDR command.
-			 * We don't actually use the serial number but we have to get
-			 * it as part of the DSP init voodoo. */
+			This is triggered by the SET_COMMPAGE_ADDR command.
+			We don't actually use the serial number but we have to
+			get it as part of the DSP init voodoo. */
 			if (read_sn(chip) < 0) {
 				DE_INIT(("load_dsp: Failed to read serial number\n"));
 				return -EIO;
@@ -517,23 +458,18 @@ static int load_dsp(echoaudio_t *chip, u16 *code)
 
 
 
-//===========================================================================
-//
-// load_firmware takes care of loading the DSP and any ASIC code.
-//
-//===========================================================================
-
-static int load_firmware(echoaudio_t *chip)
+/* load_firmware takes care of loading the DSP and any ASIC code. */
+static int load_firmware(struct echoaudio *chip)
 {
 	const struct firmware *fw;
-	int err;
+	int box_type, err;
 
 	snd_assert(chip->dsp_code_to_load && chip->comm_page, return -EPERM);
 
 	/* See if the ASIC is present and working - only if the DSP is already loaded */
 	if (chip->dsp_code) {
-		if (check_asic_status(chip) == 0)
-			return 0;
+		if ((box_type = check_asic_status(chip)) >= 0)
+			return box_type;
 		/* ASIC check failed; force the DSP to reload */
 		chip->dsp_code = NULL;
 	}
@@ -545,26 +481,25 @@ static int load_firmware(echoaudio_t *chip)
 	if (err < 0)
 		return err;
 
-	/* Load the ASIC if the DSP load succeeded. >0 means it did not fail
-	but we cannot continue (eg. we loaded the wrong firmware) */
-	if ((err = load_asic(chip)) != 0)
+	if ((box_type = load_asic(chip)) < 0)
+		return box_type;	/* error */
+
+	if ((err = restore_dsp_rettings(chip)) < 0)
 		return err;
 
-	return restore_dsp_rettings(chip);
+	return box_type;
 }
 
 
 
 /****************************************************************************
-
-  Mixer functions
-
+	Mixer functions
  ****************************************************************************/
 
 #if defined(ECHOCARD_HAS_INPUT_NOMINAL_LEVEL) || defined(ECHOCARD_HAS_OUTPUT_NOMINAL_LEVEL)
 
 /* Set the nominal level for an input or output bus (true = -10dBV, false = +4dBu) */
-static int set_nominal_level(echoaudio_t *chip, u16 index, char consumer)
+static int set_nominal_level(struct echoaudio *chip, u16 index, char consumer)
 {
 	snd_assert(index < num_busses_out(chip) + num_busses_in(chip), return -EINVAL);
 
@@ -586,9 +521,8 @@ static int set_nominal_level(echoaudio_t *chip, u16 index, char consumer)
 
 
 
-/* Set the gain for a single physical output channel (dB).
-ECHOGAIN_MINOUT <= gain <= ECHOGAIN_MAXOUT) */
-static int set_output_gain(echoaudio_t *chip, u16 channel, signed short gain)
+/* Set the gain for a single physical output channel (dB). */
+static int set_output_gain(struct echoaudio *chip, u16 channel, s8 gain)
 {
 	snd_assert(channel < num_busses_out(chip), return -EINVAL);
 
@@ -604,9 +538,8 @@ static int set_output_gain(echoaudio_t *chip, u16 channel, signed short gain)
 
 
 #ifdef ECHOCARD_HAS_MONITOR
-/* Set the monitor level from an input bus to an output bus.
-ECHOGAIN_MINOUT <= gain <= ECHOGAIN_MAXOUT) */
-static int set_monitor_gain(echoaudio_t *chip, u16 output, u16 input, signed short gain)
+/* Set the monitor level from an input bus to an output bus. */
+static int set_monitor_gain(struct echoaudio *chip, u16 output, u16 input, s8 gain)
 {
 	snd_assert(output < num_busses_out(chip) && input < num_busses_in(chip), return -EINVAL);
 
@@ -621,7 +554,7 @@ static int set_monitor_gain(echoaudio_t *chip, u16 output, u16 input, signed sho
 
 
 /* Tell the DSP to read and update output, nominal & monitor levels in comm page. */
-static int update_output_line_level(echoaudio_t *chip)
+static int update_output_line_level(struct echoaudio *chip)
 {
 	if (wait_handshake(chip))
 		return -EIO;
@@ -632,7 +565,7 @@ static int update_output_line_level(echoaudio_t *chip)
 
 
 /* Tell the DSP to read and update input levels in comm page */
-static int update_input_line_level(echoaudio_t *chip)
+static int update_input_line_level(struct echoaudio *chip)
 {
 	if (wait_handshake(chip))
 		return -EIO;
@@ -644,7 +577,7 @@ static int update_input_line_level(echoaudio_t *chip)
 
 /* set_meters_on turns the meters on or off.  If meters are turned on, the DSP
 will write the meter and clock detect values to the comm page at about 30Hz */
-static void set_meters_on(echoaudio_t *chip, char on)
+static void set_meters_on(struct echoaudio *chip, char on)
 {
 	if (on && !chip->meters_enabled) {
 		send_vector(chip, DSP_VC_METERS_ON);
@@ -659,22 +592,15 @@ static void set_meters_on(echoaudio_t *chip, char on)
 
 
 
-//===========================================================================
-//
-// Fill out an the given array using the current values in the comm page.
-//
-// Meters are written in the comm page by the DSP in this order:
-//
-// Output busses
-// Input busses
-// Output pipes (vmixer cards only)
-//
-// This function assumes there are no more than 16 in/out busses or pipes
-// (maximum is ECHO_MAXAUDIO*, but currently there no Echoaudio cards
-// have so many channels. Meters is an array [3][16][2])
-//
-//===========================================================================
-static void get_audio_meters(echoaudio_t *chip, long *meters)
+/* Fill out an the given array using the current values in the comm page.
+Meters are written in the comm page by the DSP in this order:
+ Output busses
+ Input busses
+ Output pipes (vmixer cards only)
+
+This function assumes there are no more than 16 in/out busses or pipes
+Meters is an array [3][16][2] of long. */
+static void get_audio_meters(struct echoaudio *chip, long *meters)
 {
 	int i, m, n;
 
@@ -687,6 +613,10 @@ static void get_audio_meters(echoaudio_t *chip, long *meters)
 	for (; n < 32; n++)
 		meters[n] = 0;
 
+#ifdef ECHOCARD_ECHO3G
+	m = E3G_MAX_OUTPUTS;	/* Skip unused meters */
+#endif
+
 	for (i = 0; i < num_busses_in(chip); i++, m++) {
 		meters[n++] = chip->comm_page->vu_meter[m];
 		meters[n++] = chip->comm_page->peak_meter[m];
@@ -706,13 +636,7 @@ static void get_audio_meters(echoaudio_t *chip, long *meters)
 
 
 
-//===========================================================================
-//
-// This is called after load_firmware to restore old gains, meters on, monitors, etc.
-//
-//===========================================================================
-
-static int restore_dsp_rettings(echoaudio_t *chip)
+static int restore_dsp_rettings(struct echoaudio *chip)
 {
 	int err;
 	DE_INIT(("restore_dsp_settings\n"));
@@ -764,15 +688,13 @@ static int restore_dsp_rettings(echoaudio_t *chip)
 
 
 /****************************************************************************
-
 	Transport functions
-
  ****************************************************************************/
 
 /* set_audio_format() sets the format of the audio data in host memory for
 this pipe.  Note that _MS_ (mono-to-stereo) playback modes are not used by ALSA
 but they are here because they are just mono while capturing */
-static void set_audio_format(echoaudio_t *chip, u16 pipe_index, const audioformat_t *format)
+static void set_audio_format(struct echoaudio *chip, u16 pipe_index, const struct audioformat *format)
 {
 	u16 dsp_format;
 
@@ -844,23 +766,21 @@ static void set_audio_format(echoaudio_t *chip, u16 pipe_index, const audioforma
 
 
 /* start_transport starts transport for a set of pipes.
-   The bits 1 in channel_mask specify what pipes to start. Only the bit of the
-   first channel must be set, regardless its interleave. Same for Pause and Stop. */
-static int start_transport(echoaudio_t *chip, u32 channel_mask, u32 cyclic_mask)
+The bits 1 in channel_mask specify what pipes to start. Only the bit of the
+first channel must be set, regardless its interleave.
+Same thing for pause_ and stop_ -trasport below. */
+static int start_transport(struct echoaudio *chip, u32 channel_mask, u32 cyclic_mask)
 {
 	DE_ACT(("start_transport %x\n", channel_mask));
 
-	/* Wait for the previous command to complete */
 	if (wait_handshake(chip))
 		return -EIO;
 
-	/* Write the appropriate fields in the comm page */
 	chip->comm_page->cmd_start |= cpu_to_le32(channel_mask);
 
 	if (chip->comm_page->cmd_start) {
 		clear_handshake(chip);
 		send_vector(chip, DSP_VC_START_TRANSFER);
-		/* Wait for transport to start */
 		if (wait_handshake(chip))
 			return -EIO;
 		/* Keep track of which pipes are transporting */
@@ -875,23 +795,18 @@ static int start_transport(echoaudio_t *chip, u32 channel_mask, u32 cyclic_mask)
 
 
 
-/* pause_transport pauses transport for a set of pipes */
-static int pause_transport(echoaudio_t *chip, u32 channel_mask)
+static int pause_transport(struct echoaudio *chip, u32 channel_mask)
 {
 	DE_ACT(("pause_transport %x\n", channel_mask));
 
-	/* Wait for the last command to finish */
 	if (wait_handshake(chip))
 		return -EIO;
 
-	/* Write to the comm page */
 	chip->comm_page->cmd_stop |= cpu_to_le32(channel_mask);
 	chip->comm_page->cmd_reset = 0;
 	if (chip->comm_page->cmd_stop) {
-		/* Clear the handshake and send the vector command */
 		clear_handshake(chip);
 		send_vector(chip, DSP_VC_STOP_TRANSFER);
-		/* Wait for transport to stop */
 		if (wait_handshake(chip))
 			return -EIO;
 		/* Keep track of which pipes are transporting */
@@ -907,23 +822,18 @@ static int pause_transport(echoaudio_t *chip, u32 channel_mask)
 
 
 
-/* stop_transport resets transport for a set of pipes */
-static int stop_transport(echoaudio_t *chip, u32 channel_mask)
+static int stop_transport(struct echoaudio *chip, u32 channel_mask)
 {
 	DE_ACT(("stop_transport %x\n", channel_mask));
 
-	/* Wait for the last command to finish */
 	if (wait_handshake(chip))
 		return -EIO;
 
-	/* Write to the comm page */
 	chip->comm_page->cmd_stop |= cpu_to_le32(channel_mask);
 	chip->comm_page->cmd_reset |= cpu_to_le32(channel_mask);
 	if (chip->comm_page->cmd_reset) {
-		/* Clear the handshake and send the vector command */
 		clear_handshake(chip);
 		send_vector(chip, DSP_VC_STOP_TRANSFER);
-		/* Wait for transport to stop */
 		if (wait_handshake(chip))
 			return -EIO;
 		/* Keep track of which pipes are transporting */
@@ -939,7 +849,7 @@ static int stop_transport(echoaudio_t *chip, u32 channel_mask)
 
 
 
-static inline int is_pipe_allocated(echoaudio_t *chip, u16 pipe_index)
+static inline int is_pipe_allocated(struct echoaudio *chip, u16 pipe_index)
 {
 	return (chip->pipe_alloc_mask & (1 << pipe_index));
 }
@@ -947,8 +857,8 @@ static inline int is_pipe_allocated(echoaudio_t *chip, u16 pipe_index)
 
 
 /* Stops everything and turns off the DSP. All pipes should be already
-   stopped and unallocated. */
-static int rest_in_peace(echoaudio_t *chip)
+stopped and unallocated. */
+static int rest_in_peace(struct echoaudio *chip)
 {
 	DE_ACT(("rest_in_peace() open=%x\n", chip->pipe_alloc_mask));
 
@@ -974,13 +884,11 @@ static int rest_in_peace(echoaudio_t *chip)
 
 
 /* Fills the comm page with default values */
-static int init_dsp_comm_page(echoaudio_t *chip)
+static int init_dsp_comm_page(struct echoaudio *chip)
 {
-	int i;
-
 	/* Check if the compiler added extra padding inside the structure */
-	if (offsetof(comm_page_t, midi_output) != 0xbe0) {
-		DE_INIT(("init_dsp_comm_page() - Invalid comm_page_t structure\n"));
+	if (offsetof(struct comm_page, midi_output) != 0xbe0) {
+		DE_INIT(("init_dsp_comm_page() - Invalid struct comm_page structure\n"));
 		return -EPERM;
 	}
 
@@ -992,16 +900,14 @@ static int init_dsp_comm_page(echoaudio_t *chip)
 	chip->input_clock = ECHO_CLOCK_INTERNAL;
 	chip->output_clock = ECHO_CLOCK_WORD;
 	chip->asic_loaded = FALSE;
-	memset(chip->comm_page, 0, sizeof(comm_page_t));
+	memset(chip->comm_page, 0, sizeof(struct comm_page));
 
 	/* Init the comm page */
-	chip->comm_page->comm_size = __constant_cpu_to_le32(sizeof(comm_page_t));
+	chip->comm_page->comm_size = __constant_cpu_to_le32(sizeof(struct comm_page));
 	chip->comm_page->handshake = 0xffffffff;
 	chip->comm_page->midi_out_free_count = __constant_cpu_to_le32(DSP_MIDI_OUT_FIFO_SIZE);
 	chip->comm_page->sample_rate = __constant_cpu_to_le32(44100);
 	chip->sample_rate = 44100;
-	for (i = 0; i < DSP_MAXAUDIOINPUTS; i++)
-		chip->comm_page->line_in_level[i] = 0;
 
 	/* Set line levels so we don't blast any inputs on startup */
 	memset(chip->comm_page->monitors, ECHOGAIN_MUTED, MONITOR_ARRAY_SIZE);
@@ -1012,14 +918,9 @@ static int init_dsp_comm_page(echoaudio_t *chip)
 
 
 
-/*===========================================================================
-//
-// This function initializes the several volume controls for busses and pipes.
-// This MUST be called after the DSP is up and running !
-//
-===========================================================================*/
-
-static int init_line_levels(echoaudio_t *chip)
+/* This function initializes the several volume controls for busses and pipes.
+This MUST be called after the DSP is up and running ! */
+static int init_line_levels(struct echoaudio *chip)
 {
 	int st, i, o;
 
@@ -1065,16 +966,10 @@ static int init_line_levels(echoaudio_t *chip)
 
 
 
-/*===========================================================================
-//
-// This isn't the interrupt handler itself; rather, the OS-specific layer
-// of the driver has an interrupt handler that calls this function.
-//
-// Returns -1 if the IRQ is not ours, 0 if ack, N > 0 if there are N midi data
-//
-===========================================================================*/
-
-static int service_irq(echoaudio_t *chip)
+/* This is low level part of the interrupt handler.
+It returns -1 if the IRQ is not ours, or N>=0 if it is, where N is the number
+of midi data in the input queue. */
+static int service_irq(struct echoaudio *chip)
 {
 	int st;
 
@@ -1082,7 +977,7 @@ static int service_irq(echoaudio_t *chip)
 	if (get_dsp_register(chip, CHI32_STATUS_REG) & CHI32_STATUS_IRQ) {
 		st = 0;
 #ifdef ECHOCARD_HAS_MIDI
-		/* If this was a MIDI input interrupt, get the MIDI input data */
+		/* Get and parse midi data if present */
 		if (chip->comm_page->midi_input[0])	/* The count is at index 0 */
 			st = midi_service_irq(chip);	/* Returns how many midi bytes we received */
 #endif
@@ -1098,19 +993,12 @@ static int service_irq(echoaudio_t *chip)
 
 
 /******************************************************************************
-
- Functions for opening and closing pipes
-
+	Functions for opening and closing pipes
  ******************************************************************************/
 
-/*===========================================================================
-//
-// allocate_pipes is used to reserve audio pipes for your exclusive use.
-// The call will fail if some pipes are already allocated.
-//
-===========================================================================*/
-
-static int allocate_pipes(echoaudio_t *chip, audiopipe_t *audiopipe, u16 pipe_index, u16 interleave)
+/* allocate_pipes is used to reserve audio pipes for your exclusive use.
+The call will fail if some pipes are already allocated. */
+static int allocate_pipes(struct echoaudio *chip, struct audiopipe *pipe, int pipe_index, int interleave)
 {
 	int i;
 	u32 channel_mask;
@@ -1123,10 +1011,8 @@ static int allocate_pipes(echoaudio_t *chip, audiopipe_t *audiopipe, u16 pipe_in
 
 	is_cyclic = 1;	/* This driver uses cyclic buffers only */
 
-	/* Compute channel mask for this substream */
 	for (channel_mask = i = 0; i < interleave; i++)
 		channel_mask |= 1 << (pipe_index + i);
-	/* Check if the specified pipes are already open */
 	if (chip->pipe_alloc_mask & channel_mask) {
 		DE_ACT(("allocate_pipes: channel already open\n"));
 		return -EAGAIN;
@@ -1136,41 +1022,32 @@ static int allocate_pipes(echoaudio_t *chip, audiopipe_t *audiopipe, u16 pipe_in
 	chip->pipe_alloc_mask |= channel_mask;
 	if (is_cyclic)
 		chip->pipe_cyclic_mask |= channel_mask;
-	audiopipe->pipe_index = pipe_index;
-	audiopipe->interleave = interleave;
-	audiopipe->state = PIPE_STATE_STOPPED;
+	pipe->index = pipe_index;
+	pipe->interleave = interleave;
+	pipe->state = PIPE_STATE_STOPPED;
 
 	/* The counter register is where the DSP writes the 32 bit DMA
 	position for a pipe.  The DSP is constantly updating this value as
 	it moves data. The DMA counter is in units of bytes, not samples. */
-	audiopipe->dma_counter = &chip->comm_page->position[pipe_index];
-	*audiopipe->dma_counter = 0;
+	pipe->dma_counter = &chip->comm_page->position[pipe_index];
+	*pipe->dma_counter = 0;
 	DE_ACT(("allocate_pipes: ok\n"));
 	return pipe_index;
 }
 
 
 
-/*===========================================================================
-//
-// free_pipes is, naturally, the inverse of allocate_pipes.
-//
-===========================================================================*/
-
-static int free_pipes(echoaudio_t *chip, audiopipe_t *audiopipe)
+static int free_pipes(struct echoaudio *chip, struct audiopipe *pipe)
 {
 	u32 channel_mask;
 	int i;
 
-	DE_ACT(("free_pipes: Pipe %d\n", audiopipe->pipe_index));
-	snd_assert(is_pipe_allocated(chip, audiopipe->pipe_index), return -EINVAL);
+	DE_ACT(("free_pipes: Pipe %d\n", pipe->index));
+	snd_assert(is_pipe_allocated(chip, pipe->index), return -EINVAL);
+	snd_assert(pipe->state == PIPE_STATE_STOPPED, return -EINVAL);
 
-	/* Compute channel mask for this substream */
-	for (channel_mask = i = 0; i < audiopipe->interleave; i++)
-		channel_mask |= 1 << (audiopipe->pipe_index + i);
-
-	/* Audio should be already stopped here */
-	snd_assert(audiopipe->state == PIPE_STATE_STOPPED, return -EINVAL);
+	for (channel_mask = i = 0; i < pipe->interleave; i++)
+		channel_mask |= 1 << (pipe->index + i);
 
 	chip->pipe_alloc_mask &= ~channel_mask;
 	chip->pipe_cyclic_mask &= ~channel_mask;
@@ -1180,30 +1057,29 @@ static int free_pipes(echoaudio_t *chip, audiopipe_t *audiopipe)
 
 
 /******************************************************************************
-
- Functions for managing the scatter-gather list
-
+	Functions for managing the scatter-gather list
 ******************************************************************************/
 
-static int sglist_init(echoaudio_t *chip, audiopipe_t *audiopipe)
+static int sglist_init(struct echoaudio *chip, struct audiopipe *pipe)
 {
-	audiopipe->sglist_head = 0;
-	memset(audiopipe->sgpage.area, 0, PAGE_SIZE);
-	chip->comm_page->sglist_addr[audiopipe->pipe_index].addr = cpu_to_le32(audiopipe->sgpage.addr);
+	pipe->sglist_head = 0;
+	memset(pipe->sgpage.area, 0, PAGE_SIZE);
+	chip->comm_page->sglist_addr[pipe->index].addr = cpu_to_le32(pipe->sgpage.addr);
 	return 0;
 }
 
 
 
-static int sglist_add_mapping(echoaudio_t *chip, audiopipe_t *audiopipe, dma_addr_t address, size_t length)
+static int sglist_add_mapping(struct echoaudio *chip, struct audiopipe *pipe,
+				dma_addr_t address, size_t length)
 {
-	int head = audiopipe->sglist_head;
-	sg_entry_t *list = (sg_entry_t *)audiopipe->sgpage.area;
+	int head = pipe->sglist_head;
+	struct sg_entry *list = (struct sg_entry *)pipe->sgpage.area;
 
 	if (head < MAX_SGLIST_ENTRIES - 1) {
 		list[head].addr = cpu_to_le32(address);
 		list[head].size = cpu_to_le32(length);
-		audiopipe->sglist_head++;
+		pipe->sglist_head++;
 	} else {
 		DE_ACT(("SGlist: too many fragments\n"));
 		return -ENOMEM;
@@ -1213,14 +1089,14 @@ static int sglist_add_mapping(echoaudio_t *chip, audiopipe_t *audiopipe, dma_add
 
 
 
-static inline int sglist_add_irq(echoaudio_t *chip, audiopipe_t *audiopipe)
+static inline int sglist_add_irq(struct echoaudio *chip, struct audiopipe *pipe)
 {
-	return sglist_add_mapping(chip, audiopipe, 0, 0);
+	return sglist_add_mapping(chip, pipe, 0, 0);
 }
 
 
 
-static inline int sglist_wrap(echoaudio_t *chip, audiopipe_t *audiopipe)
+static inline int sglist_wrap(struct echoaudio *chip, struct audiopipe *pipe)
 {
-	return sglist_add_mapping(chip, audiopipe, audiopipe->sgpage.addr, 0);
+	return sglist_add_mapping(chip, pipe, pipe->sgpage.addr, 0);
 }
