@@ -541,7 +541,7 @@ SET_LAYLA24_FREQUENCY_REG command.
 #define E3G_ADAT_MODE		0x1000
 #define E3G_SPDIF_OPTICAL_MODE	0x2000
 
-#define E3G_CLOCK_CLEAR_MASK		0xffffbff0
+#define E3G_CLOCK_CLEAR_MASK		0xbfffbff0
 #define E3G_DIGITAL_MODE_CLEAR_MASK	0xffffcfff
 #define E3G_SPDIF_FORMAT_CLEAR_MASK	0xfffff01f
 
@@ -560,8 +560,8 @@ SET_LAYLA24_FREQUENCY_REG command.
 #define E3G_FREQ_REG_MAX		0xffff
 
 // 3G external box types
-#define E3G_GINA3G			0x00
-#define E3G_LAYLA3G			0x10
+#define E3G_GINA3G_BOX_TYPE		0x00
+#define E3G_LAYLA3G_BOX_TYPE		0x10
 #define E3G_ASIC_NOT_LOADED		0xffff
 #define E3G_BOX_TYPE_MASK		0xf0
 
@@ -604,20 +604,15 @@ SET_LAYLA24_FREQUENCY_REG command.
 #define DSP_MIDI_OUT_FIFO_SIZE	64
 
 
-// The monitor array is a one-dimensional array; compute the offset into the array
-#define MONITOR_INDEX(Out, In)  (Out * NUM_BUSSES_IN + In)
-
-
-
 /* sg_entry is a single entry for the scatter-gather list.  The array of struct
 sg_entry struct is read by the DSP, so all values must be little-endian. */
 
 #define MAX_SGLIST_ENTRIES 512
 
-typedef struct {
+struct sg_entry {
 	u32 addr;
 	u32 size;
-} sg_entry_t;
+};
 
 
 /****************************************************************************
@@ -630,7 +625,7 @@ typedef struct {
 
  ****************************************************************************/
 
-typedef struct {					//					Base	Length
+struct comm_page {					//					Base	Length
 	u32 comm_size;					// size of this object			0x000	4
 	u32 flags;					// See Appendix A below			0x004	4
 	u32 unused;					// Unused entry				0x008	4
@@ -640,7 +635,7 @@ typedef struct {					//					Base	Length
 	u32 cmd_stop;					// Chs. to stop mask			0x018	4
 	u32 cmd_reset;					// Chs. to reset mask			0x01c	4
 	u16 audio_format[DSP_MAXPIPES];			// Chs. audio format			0x020	32*2
-	sg_entry_t sglist_addr[DSP_MAXPIPES];		// Chs. Physical sglist addrs		0x060	32*8
+	struct sg_entry sglist_addr[DSP_MAXPIPES];		// Chs. Physical sglist addrs		0x060	32*8
 	volatile u32 position[DSP_MAXPIPES];		// Positions for ea. ch.		0x160	32*4
 	volatile s8 vu_meter[DSP_MAXPIPES];		// VU meters				0x1e0	32*1
 	volatile s8 peak_meter[DSP_MAXPIPES];		// Peak meters				0x200	32*1
@@ -667,6 +662,6 @@ typedef struct {					//					Base	Length
 	u8 filler[24];					// filler				0xb88	24*1
 	s8 vmixer[VMIXER_ARRAY_SIZE];			// Vmixer levels			0xba0	64*1
 	u8 midi_output[MIDI_OUT_BUFFER_SIZE];		// MIDI output data			0xbe0	32*1
-} comm_page_t;
+};
 
 #endif // _ECHO_DSP_
