@@ -137,7 +137,7 @@ static char *kernel_deps[] = {
 /* % -> always true */
 /* # -> define */
 static char *no_cards[] = {
-	"%SOUND",
+	"%#SOUND",
 	"SOUND_PRIME",
 	"%SND",
 	"#SND_TIMER",
@@ -147,13 +147,16 @@ static char *no_cards[] = {
 	"SND_SEQUENCER",
 	"SND_MIXER_OSS",
 	"SND_PCM_OSS",
+	"SND_PCM_OSS_PLUGINS",
 	"SND_SEQUENCER_OSS",
 	"SND_OSSEMUL",
 	"SND_RTCTIMER",
 	"SND_HPET",
+	"SND_DYNAMIC_MINORS",
 	"SND_DEBUG",
 	"SND_DEBUG_MEMORY",
 	"SND_DEBUG_DETECT",
+	"SND_VERBOSE_PROCFS",
 	"SND_VERBOSE_PRINTK",
 	"SND_BIT32_EMUL",
 	"#SND_OPL3_LIB",
